import {app, BrowserWindow, dialog, globalShortcut, ipcMain, Menu, screen} from 'electron';
import * as path from 'path';
import * as fs from 'fs';
import {GameModelLoader} from "./storage/loader/GameModelLoader";
import {StoredGameModel} from "./storage/StoredGameModel";
import {ScriptAccountStorage} from "./storage/storing/ScriptAccountStoring";
import {ModelComponentFileDirectory} from "./storage/ModelComponentFileDirectory";
import {GamesystemStorage} from "./storage/storing/GamesystemStorage";

let win: BrowserWindow | null = null;
const args = process.argv.slice(1),
  serve = args.some(val => val === '--serve');

let projectDirectory = "testModel"
function createWindow(): BrowserWindow {

  const size = screen.getPrimaryDisplay().workAreaSize;

  // Create the browser window.
  win = new BrowserWindow({
    x: 0,
    y: 0,
    width: size.width,
    height: size.height,
    webPreferences: {
      nodeIntegration: true,
      allowRunningInsecureContent: (serve),
      contextIsolation: false,
    },
  });

  if (serve) {
    const debug = require('electron-debug');
    debug();

    require('electron-reloader')(module);
    win.loadURL('http://localhost:4200');
  } else {
    // Path when running electron executable
    let pathIndex = './index.html';

    if (fs.existsSync(path.join(__dirname, '../dist/index.html'))) {
       // Path when running electron in local folder
      pathIndex = '../dist/index.html';
    }

    const url = new URL(path.join('file:', __dirname, pathIndex));
    win.loadURL(url.href);
  }

  // Emitted when the window is closed.
  win.on('closed', () => {
    // Dereference the window object, usually you would store window
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    win = null;
  });

  const contextMenuTemplate = [
    {
      label: 'New',
      submenu: [
        {
          label: "Gamesystem",
          click: () => {
            win!.webContents.send('context-menu', "new-gamesystem");
          }
        },
        {
          label: "ScriptAccount",
          click: () => {
            win!.webContents.send('context-menu', "new-scriptaccount");
          }
        }
      ]

    },
    {
      label: 'Edit...',
      click: () => {
        win!.webContents.send('context-menu', "edit");
      }
    },
    {
      label: 'Delete...',
      click: () => {
        win!.webContents.send('context-menu', "delete");
      }
    }
  ]

  const contextMenu = Menu.buildFromTemplate(contextMenuTemplate);

  win.webContents.on('context-menu', (e, params) => {
    console.log("Electron: Context menu showing")
    contextMenu.popup({ window: win!, x: params.x, y: params.y });
  })

  ipcMain.on('save-model', (event, storedGameModel: StoredGameModel) => {
    recieveGameModelToStore(storedGameModel)
  })

  /*ipcMain.on('delete-component', (event, deletedComponent: DeleteModel) => {
    console.log("Delete Model: ", deletedComponent)
      deleteComponent(deletedComponent);
  })*/

  const menuTemplate = [
    {
      label: 'File',
      submenu: [
        {
          label: "New Project",
          click: () => {
            createNewProject();
          }
        },
        {
          label: "Open Project",
          click: () => {
            openProject()
          }
        },
        {
          label: "Save",
          click: () => {
            saveProject();
          }
        }
      ]
    }
  ]
  const menu = Menu.buildFromTemplate(menuTemplate);
  Menu.setApplicationMenu(menu)

  win.webContents.on('did-finish-load', () => {
    loadDevProjectAtStart()
  })

  return win;
}

try {
  // This method will be called when Electron has finished
  // initialization and is ready to create browser windows.
  // Some APIs can only be used after this event occurs.
  // Added 400 ms to fix the black background issue while using transparent window. More detais at https://github.com/electron/electron/issues/15947
  app.on('ready', () => setTimeout(createWindow, 400));

  // Quit when all windows are closed.
  app.on('window-all-closed', () => {
    // On OS X it is common for applications and their menu bar
    // to stay active until the user quits explicitly with Cmd + Q
    if (process.platform !== 'darwin') {
      app.quit();
    }
  });

  app.on('activate', () => {
    // On OS X it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (win === null) {
      createWindow();
    }
  });

  app.whenReady().then(() => {
    globalShortcut.register('CommandOrControl+S', () => {
      saveProject();
    })
  })





} catch (e) {
  // Catch Error
  // throw e;
}


function createNewProject() {

}

function openProject() {
  const selectedPaths = dialog.showOpenDialogSync(win!, {
    title: "Open Game-Model",
    properties: [
      "openDirectory",
    ]
  })

  if(selectedPaths != undefined) {
    openProjectFromFile(selectedPaths[0])
  }
}

function loadDevProjectAtStart() {
    const projectDir = path.join(process.cwd(), "testModel/")
    openProjectFromFile(projectDir)
}

function openProjectFromFile(openProjectDir: string) {
  projectDirectory = openProjectDir
  const gameModelLoader = new GameModelLoader(openProjectDir);
  const loadedProject = gameModelLoader.loadGameModel();
  win!.webContents.send("open-project", loadedProject)
}

function saveProject() {
  win!.webContents.send('get-project-data')
}

function recieveGameModelToStore(gameModel: StoredGameModel) {
  const scriptAccountStorage = new ScriptAccountStorage(path.join(projectDirectory, ModelComponentFileDirectory.SCRIPTACCOUNT_DIR_NAME))
  scriptAccountStorage.storeScriptAccounts(gameModel.storedScriptAccounts)

  const gamesystemStorage = new GamesystemStorage(path.join(projectDirectory, ModelComponentFileDirectory.GAMESYSTEM_DIR_NAME))
  gamesystemStorage.storeGamesystems(gameModel.storedGamesystems)
}

/*function deleteComponent(component: DeleteModel) {
  console.log("Delete Component")
  if(component.modeltype == ModelComponentType.SCRIPTACCOUNT) {
    DeleteTransaction.deleteScriptAccount(projectDirectory, component.componentName);
  } else if(component.modeltype === ModelComponentType.GAMESYTEM) {
    DeleteTransaction.deleteGamesystem(projectDirectory, component.componentName);
  }
}*/
