import {Component, Input, OnInit} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {MatSnackBar} from "@angular/material/snack-bar";
import {SimpleState} from "../../../../project/game-model/gamesystems/states/SimpleState";
import {SimpleGamesystem} from "../../../../project/game-model/gamesystems/SimpleGamesystem";
import {ScriptAccount} from "../../../../project/game-model/scriptAccounts/ScriptAccount";
import {ScriptAccountCondition} from "../../../../project/game-model/gamesystems/conditions/ScriptAccountCondition";

@Component({
  selector: 'app-simple-state-editor',
  templateUrl: './simple-state-editor.component.html',
  styleUrl: './simple-state-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class SimpleStateEditorComponent implements OnInit{

  @Input() states: SimpleState[] = [];
  @Input() gamesystem: SimpleGamesystem | undefined
  @Input() scriptAccounts: ScriptAccount[] = []
  dataSource = new MatTableDataSource<SimpleState>();
  displayedColumns = ["name", "initial", "edit", "delete"];
  columnsToDisplayWithExpand = [...this.displayedColumns, 'expand'];
  expandedElement: SimpleState | null = null;
  editedElement: SimpleState | null = null;

  editedStateLabelError: boolean = false;

  constructor(private snackbar: MatSnackBar) {
  }

  ngOnInit() {
    this.dataSource.data = this.states;
    this.dataSource.filterPredicate = (data: SimpleState, filter: string) => {
      return data.stateLabel.toLowerCase().includes(filter);
    }
  }

  editState(state: SimpleState) {
    if(this.editedElement === state) {
      this.editedElement = null;
    } else {
      this.editedElement = state;
    }
  }

  finishEditing() {
    if(this.isEditedStateValid()) {
      this.editedElement = null;
    }
  }

  isEditedStateValid(): boolean {
    if(this.editedElement!.stateLabel.length > 0) {
      this.editedStateLabelError = false;
      return true;
    } else {
      this.editedStateLabelError = true;
    }
    return false;
  }

  deleteState(state: SimpleState) {
    if(this.gamesystem == undefined || this.gamesystem.parentGamesystem == undefined) {
      this.gamesystem?.removeState(state);
      this.dataSource.data = this.gamesystem!.states
    } else {

    }
  }

  addState() {
    if(this.gamesystem != undefined) {
      const simpleState = this.gamesystem.createState("New State", "");
      if(simpleState != undefined) {
        this.dataSource.data = this.gamesystem.states;
        this.editedElement = simpleState;
        this.expandedElement = simpleState;
        this.onStateChange();
      }
    }
  }
  onStateChange() {
    this.gamesystem!.onModifyContent();
  }

  applyFilter(event: KeyboardEvent) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
  }

  onCreateCondition(state: SimpleState, condition: ScriptAccountCondition) {
    state.addScriptAccountCondition(condition);
  }


  deleteCondition(state: SimpleState, condition: ScriptAccountCondition) {
    state.removeScriptAccountCondition(condition.scriptAccount);
  }
}
